import sys
from protocol import IcingProtocol

RED = "\033[91m"
GREEN = "\033[92m"
YELLOW = "\033[93m"
BLUE = "\033[94m"
RESET = "\033[0m"


def main():
    protocol = IcingProtocol()

    print(f"{YELLOW}\n======================================")
    print("   Icing Protocol - Manual CLI Demo   ")
    print("======================================\n" + RESET)

    print(f"Listening on port: {protocol.local_port}")
    print(f"Your identity public key (hex): {protocol.identity_pubkey.hex()}")
    print("\nAvailable commands:")
    print("  set_peer_identity <hex_pubkey>")
    print("  connect <port>")
    print("  generate_ephemeral_keys")
    print("  send_ping")
    print("  send_handshake")
    print("  respond_ping <index> <0|1>")
    print("  generate_ecdhe <index>")
    print("  auto_responder <on|off>")
    print("  show_state")
    print("  exit\n")

    while True:
        try:
            line = input("Cmd> ").strip()
        except EOFError:
            break
        if not line:
            continue

        parts = line.split()
        cmd = parts[0].lower()

        if cmd == "exit":
            protocol.stop()
            sys.exit(0)

        elif cmd == "show_state":
            protocol.show_state()

        elif cmd == "set_peer_identity":
            if len(parts) != 2:
                print(f"{RED}Usage: set_peer_identity <hex_pubkey>{RESET}")
                continue
            protocol.set_peer_identity(parts[1])

        elif cmd == "connect":
            if len(parts) != 2:
                print(f"{RED}Usage: connect <port>{RESET}")
                continue
            try:
                port = int(parts[1])
                protocol.connect_to_peer(port)
            except ValueError:
                print(f"{RED}Invalid port.{RESET}")

        elif cmd == "generate_ephemeral_keys":
            protocol.generate_ephemeral_keys()

        elif cmd == "send_ping":
            protocol.send_ping_request()

        elif cmd == "send_handshake":
            protocol.send_handshake()

        elif cmd == "respond_ping":
            if len(parts) != 3:
                print(f"{RED}Usage: respond_ping <index> <answer_code>{RESET}")
                continue
            try:
                idx = int(parts[1])
                ac = int(parts[2])
                protocol.respond_to_ping(idx, ac)
            except ValueError:
                print(f"{RED}Index and answer_code must be integers.{RESET}")

        elif cmd == "generate_ecdhe":
            if len(parts) != 2:
                print(f"{RED}Usage: generate_ecdhe <index>{RESET}")
                continue
            try:
                idx = int(parts[1])
                protocol.generate_ecdhe(idx)
            except ValueError:
                print(f"{RED}Index must be an integer.{RESET}")

        elif cmd == "auto_responder":
            if len(parts) != 2:
                print(f"{RED}Usage: auto_responder <on|off>{RESET}")
                continue
            arg = parts[1].lower()
            if arg == "on":
                protocol.enable_auto_responder(True)
            elif arg == "off":
                protocol.enable_auto_responder(False)
            else:
                print(f"{RED}Usage: auto_responder <on|off>{RESET}")

        else:
            print(f"{RED}[ERROR]{RESET} Unknown command: {cmd}")


if __name__ == "__main__":
    main()
